Package: PolySTest
Title: PolySTest: Detection of differentially regulated features.
        Combined statistical testing for data with few replicates and
        missing values
Version: 1.7.0
Authors@R: 
    person("Veit", "Schwämmle", , "veits@bmb.sdu.dk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9708-6722"))
Description: The complexity of high-throughput quantitative omics
        experiments often leads to low replicates numbers and many
        missing values. We implemented a new test to simultaneously
        consider missing values and quantitative changes, which we
        combined with well-performing statistical tests for high
        confidence detection of differentially regulated features. The
        package contains functions to run the test and to visualize the
        results.
License: GPL-2
Encoding: UTF-8
biocViews: MassSpectrometry, Proteomics, Software,
        DifferentialExpression
BugReports: https://github.com/computproteomics/PolySTest/issues
URL: https://github.com/computproteomics/PolySTest
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Imports: fdrtool (>= 1.2.15), limma (>= 3.61.3), matrixStats (>=
        0.57.0), qvalue (>= 2.22.0), shiny (>= 1.5.0),
        SummarizedExperiment (>= 1.20.0), knitr (>= 1.33), plotly (>=
        4.9.4), heatmaply (>= 1.1.1), circlize (>= 0.4.12), UpSetR (>=
        1.4.0), gplots (>= 3.1.1), S4Vectors (>= 0.30.0), parallel (>=
        4.1.0), grDevices (>= 4.1.0), graphics (>= 4.1.0), stats (>=
        4.1.0), utils (>= 4.1.0)
Suggests: testthat (>= 3.0.0), BiocStyle
VignetteBuilder: knitr
Config/testthat/edition: 3
Depends: R (>= 4.4.0)
Config/pak/sysreqs: cmake make libmagick++-dev gsfonts libicu-dev
        libuv1-dev libssl-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 13:03:22 UTC
RemoteUrl: https://github.com/bioc/PolySTest
RemoteRef: HEAD
RemoteSha: 92e64516396520f37627f67bfafa3d5e381c3153
NeedsCompilation: no
Packaged: 2026-05-29 10:22:07 UTC; root
Author: Veit Schwämmle [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9708-6722>)
Maintainer: Veit Schwämmle <veits@bmb.sdu.dk>
Built: R 4.6.0; ; 2026-05-29 12:08:40 UTC; unix
