# Generated by roxygen2: do not edit by hand

export(FindFCandQlim)
export(FindFCandQlimAlternative)
export(MissValPDistr)
export(MissingStats)
export(MissingStatsDesign)
export(PolySTest_paired)
export(PolySTest_unpaired)
export(RPStats)
export(StatsForPermutTest)
export(calculate_qvalues)
export(check_for_polystest)
export(check_stat_names)
export(create_pairwise_comparisons)
export(get_numthreads)
export(limma_paired)
export(limma_unpaired)
export(perm_unpaired)
export(permtest_paired)
export(plotExpression)
export(plotHeatmaply)
export(plotPvalueDistr)
export(plotRegNumber)
export(plotUpset)
export(plotVolcano)
export(rp_unpaired)
export(set_mfrow)
export(ttest_paired)
export(ttest_unpaired)
export(update_conditions_with_lcs)
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(UpSetR,upset)
importFrom(circlize,circos.axis)
importFrom(circlize,circos.clear)
importFrom(circlize,circos.initialize)
importFrom(circlize,circos.par)
importFrom(circlize,circos.rect)
importFrom(circlize,circos.text)
importFrom(circlize,circos.trackPlotRegion)
importFrom(circlize,get.cell.meta.data)
importFrom(fdrtool,hc.thresh)
importFrom(gplots,plotCI)
importFrom(gplots,redblue)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,grey.colors)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(heatmaply,heatmaply)
importFrom(knitr,kable)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,topTable)
importFrom(matrixStats,colMins)
importFrom(matrixStats,rowMins)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(plotly,plotly_empty)
importFrom(qvalue,qvalue)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pgamma)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(utils,combn)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
